@@ -44,9 +44,11 @@
 #include "option_table.h"
 #include "msg.h"
 #include "manifest.h"
+#include "initramfs.h"
 
 static void print_version(void);
 static void print_help(const char* name, int is_uninstall, int advanced);
+void free_package(Package *p);
 
 
 
@@ -146,6 +148,8 @@ static Options *load_default_options(void)
     op->external_platform_json_path = DEFAULT_EGL_EXTERNAL_PLATFORM_JSON_PATH;
     op->skip_depmod = FALSE;
     op->use_systemd = NV_OPTIONAL_BOOL_DEFAULT;
+    op->rebuild_initramfs = NV_OPTIONAL_BOOL_DEFAULT;
+    op->disable_nouveau = TRUE;
 
     return op;
 
@@ -182,9 +186,30 @@ static int assign_file_type_override(Options *op, char *optarg)
 
 static int print_recommended_module_type_option(Options *op)
 {
-    /* On 535, proprietary modules is always the default */
-    nv_info_msg(NULL, "proprietary");
+    Package *p;
+    op->logging = 0;
+    op->silent = 1;
+
+    /* initialize the user interface; needed for check_nvidia_graphics_devices */
+    if (!ui_init(op)) goto failed;
+
+    if ((p = parse_manifest(op)) == NULL) goto failed;
+
+    check_for_nvidia_graphics_devices(op, p);
+
+    if (strcmp(p->kernel_module_build_directory, "kernel-open") == 0) {
+        nv_info_msg(NULL, "open");
+    }
+    else {
+        nv_info_msg(NULL, "proprietary");
+    }
+
+    free_package(p);
     return TRUE;
+
+failed:
+    ui_error(op, "Unable to determine recommended module type.");
+    return FALSE;
 }
 
 /*
@@ -234,7 +259,7 @@ static void parse_commandline(int argc, char *argv[], Options *op)
         case 'd': op->debug = TRUE; break;
         case 'i':
             op->driver_info = TRUE;
-            op->ui_str = "none";
+            op->ui.name = "none";
             break;
         case 'n': op->no_precompiled_interface = TRUE; break;
         case 'c': op->no_ncurses_color = TRUE; break;
@@ -252,10 +277,10 @@ static void parse_commandline(int argc, char *argv[], Options *op)
         case 'X': op->run_nvidia_xconfig = TRUE; break;
         case 's':
             op->silent = op->no_questions = TRUE;
-            op->ui_str = "none";
+            op->ui.name = "none";
             break;
         case 'z': op->no_nouveau_check = TRUE; break;
-        case 'Z': op->disable_nouveau = TRUE; break;
+        case 'Z': op->disable_nouveau = boolval; break;
         case 'k':
             op->kernel_name = strval;
             op->no_precompiled_interface = TRUE;
@@ -323,10 +348,12 @@ static void parse_commandline(int argc, char *argv[], Options *op)
             op->uninstall = TRUE; break;
         case SKIP_MODULE_UNLOAD_OPTION:
             op->skip_module_unload = TRUE; break;
+        case SKIP_MODULE_LOAD_OPTION:
+            op->skip_module_load = TRUE; break;
         case PROC_MOUNT_POINT_OPTION:
             op->proc_mount_point = strval; break;
         case USER_INTERFACE_OPTION:
-            op->ui_str = strval; break;
+            op->ui.name = strval; break;
         case LOG_FILE_NAME_OPTION:
             op->log_file_name = strval; break;
         case HELP_ARGS_ONLY_OPTION:
@@ -530,6 +557,13 @@ static void parse_commandline(int argc, char *argv[], Options *op)
         case PRINT_RECOMMENDED_MODULE_TYPE_OPTION:
             print_recommended_kernel_module_after = TRUE;
             break;
+        case ALLOW_INSTALLATION_WITH_RUNNING_DRIVER_OPTION:
+            op->allow_installation_with_running_driver = boolval;
+            break;
+        case REBUILD_INITRAMFS_OPTION:
+            op->rebuild_initramfs = boolval ? NV_OPTIONAL_BOOL_TRUE :
+                                              NV_OPTIONAL_BOOL_FALSE;
+            break;
         case GBM_BACKEND_DIR_OPTION:
             op->gbm_backend_dir = strval;
             break;
@@ -632,6 +666,10 @@ int main(int argc, char *argv[])
     
     if (!ui_init(op)) return 1;
 
+    if (!begin_initramfs_scan(op)) {
+        ui_log(op, "Failed to initiate initramfs scan");
+    }
+
     /* determine the concurrency level: do this early on, to allow for
      * parallelization of as much of the install as possible. */
 
@@ -694,6 +732,10 @@ int main(int argc, char *argv[])
         ret = install_from_cwd(op);
     }
 
+    if (ret) {
+        suggest_reboot(op);
+    }
+
  done:
     
     ui_close(op);
