@@ -44,6 +44,7 @@ enum {
     KERNEL_INSTALL_PATH_OPTION,
     UNINSTALL_OPTION,
     SKIP_MODULE_UNLOAD_OPTION,
+    SKIP_MODULE_LOAD_OPTION,
     PROC_MOUNT_POINT_OPTION,
     USER_INTERFACE_OPTION,
     LOG_FILE_NAME_OPTION,
@@ -116,6 +117,8 @@ enum {
     SYSTEMD_SLEEP_PREFIX_OPTION,
     SYSTEMD_SYSCONF_PREFIX_OPTION,
     GBM_BACKEND_DIR_OPTION,
+    ALLOW_INSTALLATION_WITH_RUNNING_DRIVER_OPTION,
+    REBUILD_INITRAMFS_OPTION,
     PRINT_RECOMMENDED_MODULE_TYPE_OPTION,
 };
 
@@ -158,6 +161,10 @@ static const NVGetoptOption __options[] = {
       "When uninstalling the driver, skip unloading of the NVIDIA kernel "
       "module. This option is ignored when the driver is being installed." },
 
+    { "skip-module-load", SKIP_MODULE_LOAD_OPTION, 0, NULL,
+      "Skip the test load of the NVIDIA kernel modules after the modules are "
+      "built, and skip loading them after installation is complete."},
+
     { "sanity", SANITY_OPTION, 0, NULL,
       "Perform basic sanity tests on an existing NVIDIA "
       "driver installation." },
@@ -475,11 +482,11 @@ static const NVGetoptOption __options[] = {
       "Normally, nvidia-installer aborts installation if the nouveau kernel "
       "driver is in use.  Use this option to disable this check." },
 
-    { "disable-nouveau", 'Z', 0, NULL,
-      "If the nouveau kernel module is detected by nvidia-installer, the "
-      "installer offers to attempt to disable nouveau. The default action "
-      "is to not attempt to disable nouveau; use this option to change the "
-      "default action to attempt to disable nouveau."},
+    { "disable-nouveau", 'Z', NVGETOPT_IS_BOOLEAN, NULL,
+      "nvidia-installer will attempt to disable the nouveau kernel driver "
+      "by default, if it is in use during installation. Use "
+      "'--no-disable-nouveau to prevent nvidia-installer from disabling "
+      "nouveau by default."},
 
     { "run-nvidia-xconfig", 'X', 0, NULL,
       "nvidia-installer can optionally invoke the nvidia-xconfig utility.  "
@@ -722,6 +729,22 @@ static const NVGetoptOption __options[] = {
       "recommended even if it is not included in this driver package."
     },
 
+    { "allow-installation-with-running-driver",
+      ALLOW_INSTALLATION_WITH_RUNNING_DRIVER_OPTION, NVGETOPT_IS_BOOLEAN, NULL,
+      "Proceed with installation even if an NVIDIA driver is already installed "
+      "and running."
+    },
+
+    { "rebuild-initramfs",
+      REBUILD_INITRAMFS_OPTION, NVGETOPT_IS_BOOLEAN, NULL,
+      "Rebuild the initramfs after installation is complete, regardless of the "
+      "default action recommended by nvidia-installer.  --no-rebuild-initramfs "
+      "skips rebuilding the initramfs after installation is complete.  These "
+      "options are useful for non-interactive installations when a specific "
+      "behavior is desired, regardless of what nvidia-installer would "
+      "recommend by default in an interactive installation."
+    },
+
     /* Orphaned options: These options were in the long_options table in
      * nvidia-installer.c but not in the help. */
     { "debug",                    'd', 0, NULL,NULL },
