@@ -38,6 +38,15 @@
 #include "misc.h"
 #include "files.h"
 #include "user-interface.h"
+#include "ui-status-indeterminate.h"
+
+#include "nvidia-installer-ncurses-ui.so.h"
+#if defined(NV_INSTALLER_NCURSES6)
+#include "nvidia-installer-ncurses6-ui.so.h"
+#endif
+#if defined(NV_INSTALLER_NCURSESW6)
+#include "nvidia-installer-ncursesw6-ui.so.h"
+#endif
 
 /*
  * global user interface pointer
@@ -55,19 +64,6 @@ char *__extracted_user_interface_filename = NULL;
 
 extern InstallerUI stream_ui_dispatch_table;
 
-/* pull in the user interface data arrays and sizes */
-
-extern const char ncurses_ui_array[];
-extern const int ncurses_ui_array_size;
-#if defined(NV_INSTALLER_NCURSES6)
-extern const char ncurses6_ui_array[];
-extern const int ncurses6_ui_array_size;
-#endif
-#if defined(NV_INSTALLER_NCURSESW6)
-extern const char ncursesw6_ui_array[];
-extern const int ncursesw6_ui_array_size;
-#endif
-
 /* struct describing the ui data */
 
 typedef struct {
@@ -108,13 +104,20 @@ int ui_init(Options *op)
         /* { "nvidia-installer GTK+ user interface", NULL, NULL, 0 }, */
 #if defined(NV_INSTALLER_NCURSES6)
         { "ncurses6", "nvidia-installer ncurses v6 user interface", NULL,
-          ncurses6_ui_array, ncurses6_ui_array_size },
+          _binary_nvidia_installer_ncurses6_ui_so_start,
+          _binary_nvidia_installer_ncurses6_ui_so_end - _binary_nvidia_installer_ncurses6_ui_so_start
+        },
 #endif
         { "ncurses", "nvidia-installer ncurses user interface", NULL,
-          ncurses_ui_array, ncurses_ui_array_size },
+          _binary_nvidia_installer_ncurses_ui_so_start,
+          _binary_nvidia_installer_ncurses_ui_so_end - _binary_nvidia_installer_ncurses_ui_so_start
+        },
 #if defined(NV_INSTALLER_NCURSESW6)
         { "ncursesw6", "nvidia-installer ncurses v6 user interface (widechar)",
-          NULL, ncursesw6_ui_array, ncursesw6_ui_array_size },
+          NULL,
+          _binary_nvidia_installer_ncursesw6_ui_so_start,
+          _binary_nvidia_installer_ncursesw6_ui_so_end - _binary_nvidia_installer_ncursesw6_ui_so_start
+        },
 #endif
         { "none", NULL, NULL, NULL, 0 }
     };
@@ -124,15 +127,15 @@ int ui_init(Options *op)
     __ui = NULL;
 
     if (!op->silent) {
-        if (op->ui_str) {
+        if (op->ui.name) {
             for (i = 0; i < ARRAY_LEN(ui_list); i++) {
-                if (strcmp(op->ui_str, ui_list[i].name) == 0) {
+                if (strcmp(op->ui.name, ui_list[i].name) == 0) {
                     break;
                 }
             }
 
             if (i == ARRAY_LEN(ui_list)) {
-                log_printf(op, NULL, "Invalid \"ui\" option: %s", op->ui_str);
+                log_printf(op, NULL, "Invalid \"ui\" option: %s", op->ui.name);
                 i = 0;
             }
         } else {
@@ -180,6 +183,8 @@ int ui_init(Options *op)
 
     if (!__ui->init(op, nv_format_text_rows)) return FALSE;
 
+    op->ui.indeterminate_data = indeterminate_init();
+
     /* handle some common signals */
 
     signal(SIGHUP,  ui_signal_handler);
@@ -463,6 +468,15 @@ int ui_paged_prompt (Options *op, const char *question, const char *pager_title,
 }
 
 
+/*
+ * ui_status_begin(): create a new status indicator and displays it immediately
+ *
+ * title: a title that will be displayed for the life of the status indicator.
+ * fmt, ...: a printf(3)-style message which may be replaced by other messages
+ *          during the life of the status indicator. This argument is optional
+ *          and may be passed NULL.
+ */
+
 void ui_status_begin(Options *op, const char *title, const char *fmt, ...)
 {
     char *msg;
@@ -473,11 +487,20 @@ void ui_status_begin(Options *op, const char *title, const char *fmt, ...)
  
     NV_VSNPRINTF(msg, fmt);
 
+    op->ui.status_active = TRUE;
+
     __ui->status_begin(op, title, msg);
     free(msg);
 }
 
-
+/*
+ * ui_status_update(): update the position of the status indicator
+ *
+ * percent: a number from 0.0 to 1.0 indicating the level of completion
+ * fmt, ...: replaces any previously displayed status message; note that some
+ *           UI implementations (e.g. stream) may only display the initial
+ *           message, if any, that was provided with ui_status_begin().
+ */
 
 void ui_status_update(Options *op, const float percent, const char *fmt, ...)
 {
@@ -491,8 +514,62 @@ void ui_status_update(Options *op, const float percent, const char *fmt, ...)
     free(msg);
 }
 
+struct indeterminate_args {
+    Options *op;
+    char *msg;
+};
+
+static void *indeterminate_worker(void *p)
+{
+    struct indeterminate_args *args = p;
+    Options *op = args->op;
+    char *msg = nvstrdup(args->msg);
+    IndeterminateData *id = op->ui.indeterminate_data;
+
+    while (indeterminate_get(id) == INDETERMINATE_ACTIVE) {
+        __ui->update_indeterminate(op, msg);
+    }
+
+    nvfree(msg);
+    return NULL;
+}
 
 
+/*
+ * ui_indeterminate_begin(): display an "indeterminate" status indicator for
+ * a task with an unknown completion point. This indicator will be shown until
+ * it is ended with ui_indeterminate_end();
+ *
+ * fmt, ...: same as ui_status_update()
+ */
+
+void ui_indeterminate_begin(Options *op, const char *fmt, ...)
+{
+    IndeterminateData *id = op->ui.indeterminate_data;
+    static struct indeterminate_args args;
+    char *msg;
+
+    if (!op->silent && fmt != NULL) {
+        NV_VSNPRINTF(msg, fmt);
+        args.op = op;
+        args.msg = msg;
+
+        indeterminate_begin(id, indeterminate_worker, &args);
+    }
+}
+
+/*
+ * ui_indeterminate_end(): terminate an indeterminate status indicator
+ */
+
+void ui_indeterminate_end(Options *op)
+{
+    indeterminate_end(op->ui.indeterminate_data);
+}
+
+/*
+ * ui_status_end(): finish the progress indicator created with ui_status_begin()
+ */
 void ui_status_end(Options *op, const char *fmt, ...)
 {
     char *msg;
@@ -502,6 +579,8 @@ void ui_status_end(Options *op, const char *fmt, ...)
     if (!op->silent) __ui->status_end(op, msg);
     log_printf(op, NV_BULLET_STR, "%s", msg);
     free(msg);
+
+    op->ui.status_active = FALSE;
 }
 
 
@@ -509,13 +588,18 @@ void ui_status_end(Options *op, const char *fmt, ...)
 void ui_close (Options *op)
 {
     if (__ui) __ui->close(op);
-                       
+
     if (__extracted_user_interface_filename) {
         unlink(__extracted_user_interface_filename);
     }
 
     __ui = NULL;
 
+    if (op) {
+        /* ui_close() may be called with NULL op from a signal handler */
+        indeterminate_destroy(op->ui.indeterminate_data);
+        op->ui.indeterminate_data = NULL;
+    }
 } /* ui_close() */
 
 
